import re

import os

from oscars_mgmnt.parser.es.es import ElasticsearchParser


class Sentinel3_Parser(ElasticsearchParser):

    def __init__(self, template=None, es_source_host=None, es_source_index=None):
        super().__init__('CGS_S3.template' if template is None else template, es_source_host, es_source_index)

    def init_dict(self):
        super().init_dict()
        self.dict_info.update(
            {
                "cogProcessingDate": "properties.processing_status.cog_processing_status.processing_end"
            }
        )

    def fix_products(self, fileobject):
        for info in list(fileobject.datas):
            name = os.path.basename(info.name)
            name = name.split('.')[0]
            info.title = os.path.basename(name)
            info.href = "file:///" + info.name
            info.conformsTo = "http://www.opengis.net/def/crs/EPSG/0/4326"


    def specific_treatment(self, product_file_info, values):
        uid = super().specific_treatment(product_file_info, values)
        self.fix_geometry(values, simplify=False)

        values['platformserialid'] = uid.split("_")[0]
        values['conformsto'] = "http://www.opengis.net/def/crs/EPSG/0/4326"
        continent = values['title'][64:81]
        continent = continent.rstrip('_')
        values['productGroupId'] = continent

        if "SY_2_V10___" in values["producttype"]:
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S3_SY_2_V10_V1"
            values['compositeType'] = "10-DAILY"
            #values.pop('platformserialid', None)
        elif "SY_2_VG1___" in values["producttype"]:
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S3_SY_2_VG1_V1"
            values['compositeType'] = "DAILY"
            #values.pop('platformserialid', None)

        return uid


class Sentinel3LstParser(ElasticsearchParser):
    """ Parser used to ingest Sentinel3 LST composites."""
    # S3B_LST_3_S1_X35Y04_20240621_1KM_LST_V100
    pattern_lst_s1 = re.compile(r"S3._LST_3_S1_X\d{2}Y\d{2}_\d{8}_1KM_LST_V\d{3}$")
    def __init__(self, template=None, es_source_host=None, es_source_index=None):
        super().__init__('CGS_S3_LST.template' if template is None else template, es_source_host, es_source_index)
        self.regex_identifier = re.compile(r"(\d{8})")

    def init_dict(self):
        super().init_dict()
        self.dict_info.update(
            {
                "lstProcessingDate": "properties.processing_status.processing_end"
            }
        )

    def get_product_identifier(self, product_file_info):
        # Extract the product_path
        product_path = product_file_info.product_path

        # Use regex to capture the filename without the extension
        filename = re.search(r'/([^/]+)\.tif$', product_path)
        if filename:
            product_identifier = filename.group(1)
            # Replace "S3A" or "S3B" with "S3"
            product_identifier = re.sub(r'^S3[AB]', 'S3', product_identifier)

            print("Product Identifier:", product_identifier)
        else:
            print("Filename not found in the path.")

        return product_identifier

    def build_new_product_identifier(self, product_file_info):
        # Extract the product_path
        product_path = product_file_info.product_path

        # Use regex to capture the filename without the extension
        filename = re.search(r'/([^/]+)\.tif$', product_path)
        if filename:
            product_identifier = filename.group(1)

            print("Product Identifier:", product_identifier)
        else:
            print("Filename not found in the path.")

        return product_identifier

    def fix_products(self, fileobject):
        for info in list(fileobject.datas):
            name = os.path.basename(info.name)
            name = name.split('.')[0]
            info.title = os.path.basename(name)
            info.href = "file:///" + info.name
            info.conformsTo = "http://www.opengis.net/def/crs/EPSG/0/4326"


    def specific_treatment(self, product_file_info, values):
        values['identifier'] = self.build_new_product_identifier(product_file_info)
        values['id'] = values['identifier']
        values['title'] = values['identifier']
        uid = super().specific_treatment(product_file_info, values)
        # self.fix_geometry(values, simplify=False)

        values['platformserialid'] = uid.split("_")[0]
        values['conformsto'] = "http://www.opengis.net/def/crs/EPSG/0/4326"
        # continent = values['title'][64:81]
        # continent = continent.rstrip('_')
        # values['productGroupId'] = continent

        # S3B_LST_3_S1_X35Y04_20240621_1KM_LST_V100
        splits = self.get_product_identifier(product_file_info).split("_")
        tile_id = splits[4]

        # Extract the x and y coordinates as integers
        tile_x = int(tile_id[1:3])  # Skips the first character ('X') and takes the next two characters
        tile_y = int(tile_id[4:6])
        bbox = self.calculate_bbox(tile_x, tile_y)
        values['westbbox'] = bbox['west']
        values['eastbbox'] = bbox['east']
        values['northbbox'] = bbox['north']
        values['southbbox'] = bbox['south']
        # self.fix_geometry(values, simplify=False)

        if "S3_LST_3_S10" in values["producttype"]:
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S3_LST_3_S10_V1"
            values['compositeType'] = "10-DAILY"
            #values.pop('platformserialid', None)
        elif "S3_LST_3_S1" in values["producttype"]:
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S3_LST_3_S1_V1"
            values['compositeType'] = "DAILY"
            #values.pop('platformserialid', None)

        return uid

    def fix_id(self, values):
        """ Fix the ID of the product by prepending the parent identifier
        :param values: dictionary containing the values, must include the product identifier and parent identifier
        :type values: dict
        """
        values['id'] = values['parentidentifier'] + ":" + values['identifier']

    def get_metadata(self, product_identifier):
        # S3B_LST_3_S1_X35Y04_20240621_1KM_LST_V100
        splits = product_identifier.split("_")
        #  splits[7] + "_" +
        print("product_identifier :: " + product_identifier)
        doc_id=splits[0] + "_" + splits[1] + "_" + splits[2] + "_" + splits[3] + "_" + splits[5] + "_"  + splits[6] + "_" +splits[8]
        print("doc_id :: " + doc_id)
        return super().get_metadata(doc_id)

    def calculate_bbox(self, tile_x: int, tile_y: int) -> dict:
        """
        Calculate the bounding box of the tile product.

        :param tile_x: tile x from product
        :param tile_y: tile y from product
        :return: dictionary containing the west, south, east and north values
        """
        # tile_x = int(tile_index_x)
        # tile_y = int(tile_index_y)

        y_start = 75.0
        x_start = -180.0

        west = x_start + 10 * tile_x
        east = west + 10
        north = y_start - 10 * tile_y
        south = north - 10

        assert -180 <= west < east <= 180
        assert 75 >= north > south >= -65

        return {
            "west": west,
            "east": east,
            "north": north,
            "south": south
        }

